@@ -13,12 +13,12 @@ tune_params = ['width', 'fast', 'slow', 'signal', 'length', 'max_lookback', 'min
                'k_period', 'timeperiod', 'timeperiod1', 'timeperiod2', 'timeperiod3', 'slowlimit', 'fastlimit',
                'minperiod', 'maxperiod', 'acceleration', 'accelerationlong', 'accelerationshort', 'signalperiod',
                'slowperiod', 'signalperiod', 'fastperiod', 'fastk_period', 'slowk_period', 'slowd_period',
-               'fastd_period', 'k', 'd', 'smooth_k']
+               'fastd_period', 'k', 'd', 'smooth_k', 'p']
 
 talib_indicators = ['tta.BBANDS', 'tta.DEMA', 'tta.EMA', 'tta.HT_TRENDLINE', 'tta.KAMA', 'tta.MA',
                         'tta.MIDPOINT', 'tta.MIDPRICE', 'tta.SAR', 'tta.SAREXT', 'tta.SMA', 'tta.T3', 'tta.TEMA',
                         'tta.TRIMA',
-                        'tta.WMA', 'tta.ADX', 'tta.ADXR', 'tta.APO', 'tta.AROON:1', 'tta.AROONOSC', 'tta.BOP',
+                        'tta.WMA', 'tta.ADX', 'tta.ADXR', 'tta.APO', 'tta.AROON', 'tta.AROONOSC', 'tta.BOP',
                         'tta.CCI', 'tta.CMO',
                         'tta.DX', 'tta.MACD', 'tta.MACDEXT', 'tta.MACDFIX', 'tta.MFI', 'tta.MINUS_DI', 'tta.MINUS_DM',
                         'tta.MOM',
@@ -54,27 +54,216 @@ talib_indicators = ['tta.BBANDS', 'tta.DEMA', 'tta.EMA', 'tta.HT_TRENDLINE', 'tt
                         'tta.CDLTHRUSTING', 'tta.CDLTRISTAR',
                         'tta.CDLUNIQUE3RIVER', 'tta.CDLUPSIDEGAP2CROWS', 'tta.CDLXSIDEGAP3METHODS', 'tta.LINEARREG',
                         'tta.LINEARREG_ANGLE', 'tta.LINEARREG_INTERCEPT', 'tta.LINEARREG_SLOPE', 'tta.STDDEV',
-                        'tta.TSF', 'tta.VAR',]
-
-pandas_ta_indicators = ['pta.ebsw', 'pta.stdev', 'pta.mad', 'pta.quantile', 'pta.skew', 'pta.variance', 'pta.zscore', 'pta.median', 'pta.kurtosis',
-    'pta.entropy', 'pta.eri', 'pta.bias', 'pta.cg', 'pta.willr', 'pta.psl', 'pta.rvgi', 'pta.kdj', 'pta.smi', 'pta.fisher', 'pta.macd', 'pta.mom', 'pta.stochrsi',
-    'pta.apo', 'pta.squeeze_pro', 'pta.inertia', 'pta.cmo', 'pta.slope', 'pta.roc', 'pta.rsx', 'pta.dm', 'pta.cci', 'pta.cti', 'pta.er', 'pta.uo', 'pta.bop', 'pta.cfo',
-    'pta.stc', 'pta.tsi', 'pta.ao', 'pta.stoch', 'pta.qqe', 'pta.coppock', 'pta.rsi', 'pta.trix', 'pta.ppo', 'pta.pgo', 'pta.squeeze', 'pta.brar', 'pta.td_seq', 'pta.kst',
-    'pta.pvo', 'pta.amat', 'pta.adx', 'pta.vortex', 'pta.qstick', 'pta.psar', 'pta.decay', 'pta.cksp', 'pta.chop', 'pta.ttm_trend', 'pta.vhf',
-    'pta.aroon', 'pta.dpo', 'pta.donchian', 'pta.accbands', 'pta.ui', 'pta.hwc', 'pta.natr', 'pta.atr', 'pta.true_range', 'pta.massi', 'pta.thermo', 'pta.aberration', 'pta.kc',
-    'pta.bbands', 'pta.pdist', 'pta.rvi', 'pta.pvt', 'pta.efi', 'pta.adosc', 'pta.aobv', 'pta.kvo', 'pta.pvi', 'pta.nvi', 'pta.vp', 'pta.mfi', 'pta.obv', 'pta.pvol', 'pta.eom',
-    'pta.pvr', 'pta.ad', 'pta.cmf']
-
-finta_indicatrs = ['fta.SMA', 'fta.SMM', 'fta.SSMA', 'fta.EMA', 'fta.DEMA', 'fta.TEMA', 'fta.TRIMA', 'fta.TRIX',
-                   'fta.VAMA', 'fta.ER', 'fta.KAMA', 'fta.ZLEMA', 'fta.WMA', 'fta.HMA', 'fta.EVWMA', 'fta.VWAP',
-                   'fta.SMMA', 'fta.MACD', 'fta.PPO', 'fta.VW_MACD', 'fta.EV_MACD', 'fta.MOM', 'fta.ROC', 'fta.RSI',
-                   'fta.IFT_RSI', 'fta.TR', 'fta.ATR', 'fta.SAR', 'fta.BBANDS', 'fta.BBWIDTH', 'fta.MOBO',
-                   'fta.PERCENT_B', 'fta.KC', 'fta.DO', 'fta.DMI', 'fta.ADX', 'fta.PIVOT', 'fta.PIVOT_FIB',
-                   'fta.STOCH', 'fta.STOCHD', 'fta.STOCHRSI', 'fta.WILLIAMS', 'fta.UO', 'fta.AO', 'fta.MI',
-                   'fta.VORTEX', 'fta.KST', 'fta.TSI', 'fta.TP', 'fta.ADL', 'fta.CHAIKIN', 'fta.MFI', 'fta.OBV',
-                   'fta.WOBV', 'fta.VZO', 'fta.PZO', 'fta.EFI', 'fta.CFI', 'fta.EBBP', 'fta.EMV', 'fta.CCI',
-                   'fta.COPP', 'fta.BASP', 'fta.BASPN', 'fta.CMO', 'fta.CHANDELIER', 'fta.WTO', 'fta.FISH', 'fta.APZ',
-                   'fta.SQZMI', 'fta.VPT', 'fta.FVE', 'fta.VFI', 'fta.MSD', 'fta.STC']
+                        'tta.TSF', 'tta.VAR']
 
+pandas_ta_indicators = [
+    'pta.alma',
+    'pta.dema',
+    'pta.ema',
+    'pta.fwma',
+    'pta.hilo',
+    'pta.hl2',
+    'pta.hlc3',
+    # 'pta.hma',  # does not accept low ranges
+    'pta.hwma',
+     # 'pta.ichimoku',  # data leak
+    'pta.jma',
+    'pta.kama',
+    'pta.linreg',
+    'pta.mcgd',
+    'pta.midpoint',
+    'pta.midprice',
+    'pta.ohlc4',
+    'pta.pwma',
+    'pta.rma',
+    'pta.sinwma',
+    'pta.sma',
+    'pta.ssf',
+    'pta.supertrend',
+    'pta.swma',
+    'pta.t3',
+    'pta.tema',
+    'pta.trima',
+    'pta.vidya',
+    'pta.vwap',
+    'pta.wcp',
+    'pta.wma',
+    'pta.zlma',
+    'pta.decreasing',
+    'pta.increasing',
+    'pta.stdev',
+    'pta.mad',
+    'pta.quantile',
+    'pta.skew',
+    'pta.variance',
+    'pta.zscore',
+    'pta.median',
+    'pta.kurtosis',
+    'pta.entropy',
+    'pta.eri',
+    'pta.bias',
+    'pta.cg',
+    'pta.willr',
+    'pta.psl',
+    'pta.rvgi',
+    'pta.kdj',
+    'pta.smi',
+    'pta.fisher',
+    'pta.macd',
+    'pta.mom',
+    'pta.stochrsi',
+    'pta.apo',
+    'pta.squeeze_pro',
+    'pta.inertia',
+    'pta.cmo',
+    'pta.slope',
+    'pta.roc',
+    'pta.rsx',
+    'pta.dm',
+    'pta.cci',
+    'pta.cti',
+    'pta.er',
+    'pta.uo',
+    'pta.bop',
+    'pta.cfo',
+    'pta.stc',
+    'pta.tsi',
+    'pta.ao',
+    'pta.stoch',
+    'pta.qqe',
+    'pta.coppock',
+    'pta.rsi',
+    'pta.trix',
+    'pta.ppo',
+    'pta.pgo',
+    'pta.squeeze',
+    'pta.brar',
+    'pta.td_seq',
+    'pta.kst',
+    'pta.pvo',
+    'pta.amat',
+    'pta.adx',
+    'pta.vortex',
+    'pta.qstick',
+    'pta.psar',
+    'pta.decay',
+    'pta.cksp',
+    'pta.chop',
+    'pta.ttm_trend',
+    'pta.vhf',
+    'pta.aroon',
+    # 'pta.dpo',  # data leak
+    'pta.donchian',
+    'pta.accbands',
+    'pta.ui',
+    'pta.hwc',
+    'pta.natr',
+    'pta.atr',
+    'pta.true_range',
+    'pta.massi',
+    'pta.thermo',
+    'pta.aberration',
+    'pta.kc',
+    'pta.bbands',
+    'pta.pdist',
+    'pta.rvi',
+    'pta.pvt',
+    'pta.efi',
+    'pta.adosc',
+    'pta.aobv',
+    'pta.kvo',
+    'pta.pvi',
+    'pta.nvi',
+    'pta.vp',
+    'pta.mfi',
+    'pta.obv',
+    'pta.pvol',
+    'pta.eom',
+    'pta.pvr',
+    'pta.ad',
+    'pta.cmf'
+]
 
+finta_indicatrs = [
+    'fta.SMA',
+    'fta.SMM',
+    'fta.SSMA',
+    'fta.EMA',
+    'fta.DEMA',
+    'fta.TEMA',
+    'fta.TRIMA',
+    'fta.TRIX',
+    'fta.VAMA',
+    'fta.ER',
+    'fta.KAMA',
+    'fta.ZLEMA',
+    'fta.WMA',
+    'fta.HMA',
+    'fta.EVWMA',
+    'fta.VWAP',
+    'fta.SMMA',
+    # 'fta.FRAMA',  # Requires even parameters (not implemented)
+    'fta.MACD',
+    'fta.PPO',
+    'fta.VW_MACD',
+    'fta.EV_MACD',
+    'fta.MOM',
+    'fta.ROC',
+    'fta.RSI',
+    'fta.IFT_RSI',
+    'fta.TR',
+    'fta.ATR',
+    'fta.SAR',
+    'fta.BBANDS',
+    'fta.BBWIDTH',
+    'fta.MOBO',
+    'fta.PERCENT_B',
+    'fta.KC',
+    'fta.DO',
+    'fta.DMI',
+    'fta.ADX',
+    'fta.PIVOT',
+    'fta.PIVOT_FIB',
+    'fta.STOCH',
+    'fta.STOCHD',
+    'fta.STOCHRSI',
+    'fta.WILLIAMS',
+    'fta.UO',
+    'fta.AO',
+    'fta.MI',
+    'fta.VORTEX',
+    'fta.KST',
+    'fta.TSI',
+    'fta.TP',
+    'fta.ADL',
+    'fta.CHAIKIN',
+    'fta.MFI',
+    'fta.OBV',
+    'fta.WOBV',
+    'fta.VZO',
+    'fta.PZO',
+    'fta.EFI',
+    'fta.CFI',
+    'fta.EBBP',
+    'fta.EMV',
+    'fta.CCI',
+    'fta.COPP',
+    'fta.BASP',
+    'fta.BASPN',
+    'fta.CMO',
+    'fta.CHANDELIER',
+    'fta.QSTICK',
+    # 'fta.TMF',  # Not implemented
+    'fta.WTO',
+    'fta.FISH',
+    'fta.ICHIMOKU',
+    'fta.APZ',
+    'fta.SQZMI',
+    'fta.VPT',
+    'fta.FVE',
+    'fta.VFI',
+    'fta.MSD',
+    'fta.STC',
+    # 'fta.WAVEPM'  # No attribute error
+]
 
