@@ -12,7 +12,7 @@ from timeit import default_timer as timer
 from tuneta.utils import col_name
 from tuneta.utils import distance_correlation
 from yellowbrick.cluster import KElbowVisualizer
-from joblib import delayed, Parallel
+from tuneta.utils import remove_consecutive_duplicates_and_nans
 import json
 import warnings
 warnings.filterwarnings("ignore")
@@ -23,11 +23,23 @@ def trial_results(X, function, trial, sym=None):
     if sym:
         X = X.droplevel(1)
     try:
-        res = eval(function) # Eval contains reference to best trial (in argument) to re-use original parameters
+        res = eval(function)  # Eval contains reference to best trial (in argument) to re-use original parameters
     except Exception as e:
         raise Exception(e)
     if isinstance(res, tuple):
-        res = pd.DataFrame(res).T
+        try:
+            res = pd.DataFrame(res).T
+        except Exception as e:
+            print("Error:")
+            print(f"Function: {function}")
+            print(f"X Length:  {len(X)}")
+            for k, v in enumerate(res):
+                u, c = np.unique(v.index, return_counts=True)
+                dup = u[c > 1]
+                print(f"Series {k} duplicates: {len(dup)}")
+                print(v)
+            raise Exception(e)
+
     res = pd.DataFrame(res, index=X.index)  # Ensure result aligns with X
     if sym:
         res['sym'] = sym
@@ -131,23 +143,25 @@ def _objective(self, trial, X, y):
         res = pd.concat(res, axis=0).sort_index()
     else:
         res = eval_res(X, self.function, self.idx, trial)
-    # except:
-        # raise RuntimeError(f"Optuna execution error: {self.function}")
 
     # y may be a subset of X, so reduce result to y and convert to series
     res_y = res.reindex(y.index).iloc[:, 0].replace([np.inf, -np.inf], np.nan)
 
+    if self.remove_consecutive_duplicates:
+        res_y = remove_consecutive_duplicates_and_nans(res_y)
+
     # Obtain distance correlation
-    if sum(np.isnan(res_y)) >= len(res_y)*.9:  # If mostly nans in result, return nan correlation
+    # Ensure results and target are aligned with target by index
+    res_tgt = pd.concat([res_y, y], axis=1)
+    res_tgt.columns = ['results', 'target']
+
+    # Measure Correlation
+    fvi = res_tgt['results'].first_valid_index()
+    if fvi is None:
         correlation = np.nan
     else:
-        # Ensure results and target are aligned with target by index
-        res_tgt = pd.concat([res_y, y], axis=1)
-        res_tgt.columns = ['results', 'target']
-
-        # Measure Correlation
-        fvi = res_tgt['results'].first_valid_index()
         res_tgt = res_tgt[res_tgt.index >= fvi]
+        res_tgt.dropna(inplace=True)
         correlation = distance_correlation(np.array(res_tgt.target), np.array(res_tgt.results))
 
     # Save results
@@ -158,12 +172,12 @@ def _objective(self, trial, X, y):
 
 
 class Optimize():
-    def __init__(self, function, n_trials=100, n_jobs=1):
+    def __init__(self, function, n_trials=100, remove_consecutive_duplicates=False):
         self.function = function
         self.n_trials = n_trials
-        self.n_jobs = n_jobs
+        self.remove_consecutive_duplicates = remove_consecutive_duplicates
 
-    def fit(self, X, y, idx=0, verbose=False, early_stop=None, n_jobs=1):
+    def fit(self, X, y, idx=0, verbose=False, early_stop=None):
         """
         Optimize a technical indicator
         :param X: Historical dataset
@@ -193,7 +207,7 @@ class Optimize():
         # Start optimization trial
         try:
             self.study.optimize(lambda trial: _objective(self, trial, X, y),
-                n_trials=self.n_trials, callbacks=[_early_stopping_opt])
+                n_trials=self.n_trials, callbacks=[_early_stopping_opt], n_jobs=1)
 
         # Early stopping (not officially supported by Optuna)
         except optuna.exceptions.OptunaError:
