@@ -22,17 +22,19 @@ from datetime import datetime
 
 # Distance correlation
 def dc(p0, p1):
-    return distance_correlation(np.array(p0).astype(float), np.array(p1).astype(float))
+    df = pd.concat([p0, p1], axis=1).dropna()
+    res = distance_correlation(np.array(df.iloc[:, 0]).astype(float), np.array(df.iloc[:, 1]).astype(float))
+    return res
 
 
 class TuneTA():
 
-    def __init__(self, n_jobs=multiprocessing.cpu_count() - 2, verbose=False):
+    def __init__(self, n_jobs=1, verbose=False):
         self.fitted = []
         self.n_jobs = n_jobs
         self.verbose = verbose
 
-    def fit(self, X, y, trials=5, indicators=['tta'], ranges=[(3, 180)], early_stop=99999):
+    def fit(self, X, y, trials=5, indicators=['tta'], ranges=[(3, 180)], early_stop=99999, min_target_correlation=0.001, remove_consecutive_duplicates=False):
         """
         Optimize indicator parameters to maximize correlation
         :param X: Historical dataset
@@ -49,6 +51,9 @@ class TuneTA():
         if not isinstance(X.index.get_level_values(0)[0], datetime):
             raise ValueError("Index must be of type datetime")
 
+        if len(X) != len(y):
+            raise ValueError("Length of X and y must be identical")
+
         self.fitted = []  # List containing each indicator completed study
         X.columns = X.columns.str.lower()  # columns must be lower case
         pool = ProcessPool(nodes=self.n_jobs)  # Set parallel cores
@@ -122,10 +127,10 @@ class TuneTA():
 
                 # Only optimize indicators that contain tunable parameters
                 if suggest:
-                    self.fitted.append(pool.apipe(Optimize(function=fn, n_trials=trials, n_jobs=self.n_jobs).fit, X, y, idx=idx,
+                    self.fitted.append(pool.apipe(Optimize(function=fn, n_trials=trials, remove_consecutive_duplicates=remove_consecutive_duplicates).fit, X, y, idx=idx,
                         verbose=self.verbose, early_stop=early_stop))
                 else:
-                    self.fitted.append(pool.apipe(Optimize(function=fn, n_trials=1, n_jobs=self.n_jobs).fit, X, y, idx=idx,
+                    self.fitted.append(pool.apipe(Optimize(function=fn, n_trials=1, remove_consecutive_duplicates=remove_consecutive_duplicates).fit, X, y, idx=idx,
                         verbose=self.verbose, early_stop=early_stop))
 
         # Blocking wait to retrieve results
@@ -134,20 +139,34 @@ class TuneTA():
         # Fits must contain best trial data
         self.fitted = [f for f in self.fitted if len(f.study.user_attrs) > 0]
 
-        # Remove any fits with zero correlation
-        self.fitted = [f for f in self.fitted if f.study.user_attrs['best_trial'].user_attrs['correlation'] > 0]
+        # Remove any fits with less than minimum target correlation
+        self.fitted = [f for f in self.fitted if f.study.user_attrs['best_trial'].user_attrs['correlation'] > min_target_correlation]
 
         # Order fits by correlation (Descending)
         self.fitted = sorted([f for f in self.fitted], key=lambda x:x.study.user_attrs['best_trial'].value, reverse=True)
 
-    def prune(self, max_correlation=.7):
+    def prune(self, max_inter_correlation=.7, top_prior=99999, top_post=99999):
         """
         Select most correlated with target, least intercorrelated
         :param top: Selects top x most correlated with target
         :param studies: From top x, keep y least intercorelated
         :return:
         """
-        if not hasattr(self, 'f_corr'):
+
+        fit_count = len(self.fitted)
+
+        # Create feature correlation dataframe and remove duplicates
+        feature_correlation = [[f.study.user_attrs['name'], f.study.user_attrs['best_trial'].user_attrs['correlation']] for f in self.fitted]
+        feature_correlation = pd.DataFrame(feature_correlation).sort_values(by=1, ascending=False)
+        feature_correlation = feature_correlation.drop_duplicates(subset=0, keep="first")  # Duplicate indicators
+        feature_correlation[1] = feature_correlation[1].round(4)
+        feature_correlation = feature_correlation.drop_duplicates(subset=[1], keep="first")  # Duplicate correlation
+
+        # Filter top correlated features
+        feature_correlation = feature_correlation.head(top_prior)
+        self.fitted = [f for i, f in enumerate(self.fitted) if i in feature_correlation.index]
+
+        if not hasattr(self, 'f_corr') or fit_count != len(self.fitted):
             self.features_corr()
 
         # Iteratively removes least fit individual of most correlated pairs of studies
@@ -158,7 +177,7 @@ class TuneTA():
 
         most_correlated = np.unravel_index(np.argmax(correlations), correlations.shape)
         correlation = correlations[most_correlated[0], most_correlated[1]]
-        while correlation > max_correlation:
+        while correlation > max_inter_correlation:
             most_correlated = np.unravel_index(np.argmax(correlations), correlations.shape)
             worst = max(most_correlated)
             components.pop(worst)
@@ -169,7 +188,7 @@ class TuneTA():
             correlation = correlations[most_correlated[0], most_correlated[1]]
 
         # Remove most correlated fits
-        self.fitted = [self.fitted[i] for i in components]
+        self.fitted = [self.fitted[i] for i in components][:top_post]
 
         # Recalculate correlation of fits
         self.target_corr()
@@ -202,7 +221,6 @@ class TuneTA():
         res = pd.concat(self.result, axis=1)
         return res
 
-
     def target_corr(self):
         fns = []  # Function names
         cor = []  # Target Correlation
@@ -228,8 +246,7 @@ class TuneTA():
 
         # Inter Correlation
         pair_order_list = itertools.combinations(features, 2)
-        correlations = Parallel(n_jobs=self.n_jobs)(delayed(dc)(p[0], p[1]) for p in pair_order_list)  # Parallelize correlation calculation
-        # correlations = [distance_correlation(p[0], p[1]) for p in pair_order_list]
+        correlations = Parallel(n_jobs=self.n_jobs)(delayed(dc)(p[0], p[1]) for p in pair_order_list)
         correlations = squareform(correlations)
         self.f_corr = pd.DataFrame(correlations, columns=fns, index=fns)
 
@@ -252,7 +269,7 @@ class TuneTA():
         df = pd.DataFrame({'Indicator': inds, 'Times': times}).sort_values(by='Times', ascending=False)
         print(tabulate(df, headers=df.columns, tablefmt="simple"))
 
-    def prune_df(self, X, y, max_correlation=.7, report=True):
+    def prune_df(self, X, y, min_target_correlation=.05, max_inter_correlation=.7, report=True, top_prior=99999, top_post=99999):
         if X.isna().any().any() or y.isna().any():
             raise ValueError("X and y cannot contain missing values")
 
@@ -262,7 +279,10 @@ class TuneTA():
         target_correlation = pd.DataFrame(tc, index=names, columns=['Correlation']).sort_values(by=['Correlation'], ascending=False)
 
         # Columns greater than 0 correlation
-        target_correlation = target_correlation[target_correlation.Correlation > 0]
+        target_correlation = target_correlation[target_correlation.Correlation > min_target_correlation]
+
+        # Intercorrelation pruning is expensive.
+        target_correlation = target_correlation.head(top_prior)
 
         if report:
             print("\nIndicator Correlation to Target:\n")
@@ -278,7 +298,7 @@ class TuneTA():
         indices = list(range(len(correlations)))
         most_correlated = np.unravel_index(np.argmax(correlations), correlations.shape)
         correlation = correlations[most_correlated[0], most_correlated[1]]
-        while correlation > max_correlation:
+        while correlation > max_inter_correlation:
             most_correlated = np.unravel_index(np.argmax(correlations), correlations.shape)
             worst = max(most_correlated)
             components.pop(worst)
@@ -289,13 +309,22 @@ class TuneTA():
             correlation = correlations[most_correlated[0], most_correlated[1]]
 
         # Get columns of features to keep
-        columns = columns[components]
+        if len(columns):
+            columns = columns[components]
 
-        # Report intercorrelation
-        if report:
-            correlations = pd.DataFrame(correlations, columns=columns, index=columns)
-            print("\nIntercorrelation after prune:\n")
-            print(tabulate(correlations, headers=correlations.columns, tablefmt="simple"))
+            # Report intercorrelation
+            if report:
+                correlations = pd.DataFrame(correlations, columns=columns, index=columns)
+                print("\nIntercorrelation after prune:\n")
+                print(tabulate(correlations, headers=correlations.columns, tablefmt="simple"))
+
+            # Correlations to target
+            X = X[columns]
+            tc = [distance_correlation(np.array(x[1]), np.array(y)) for x in X.iteritems()]
+            names = [x[0] for x in X.iteritems()]
+            target_correlation = pd.DataFrame(tc, index=names, columns=['Correlation']).sort_values(by=['Correlation'], ascending=False)
+            target_correlation = target_correlation.head(top_post)
+            columns = target_correlation.index.values
 
         return columns
 
